# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2011 Eficent (<http://www.eficent.com/>)
#              Jordi Ballester Alomar <jordi.ballester@eficent.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import tools
from osv import fields, osv
from tools.translate import _

class task(osv.osv):
    _inherit = 'project.task'
    
    def _check_child_task(self, cr, uid, ids, context=None):
        pass
    
    def _check_child_task_done(self, cr, uid, ids, context=None):
        if context == None:
            context = {}
        tasks = self.browse(cr, uid, ids, context=context)
        for it_task in tasks:
            if it_task.parent_ids:
                for parent in it_task.parent_ids:
                    if parent.state in ['draft', 'open', 'pending']:
                        raise osv.except_osv(_("Warning !"),
                                             _("Predecessor task still open.\nPlease cancel or complete predecessor task first."))
        return True
    
    def action_close(self, cr, uid, ids, context=None):
        ret = super(task, self).action_close(cr, uid, ids, context=context)
        self._check_child_task_done(cr, uid, ids, context=context)
        return ret
        
    def case_close(self, cr, uid, ids, context=None):
        ret = super(task, self).case_close(cr, uid, ids, context=context)
        self.advance_childs_if_possible(cr, uid, ids, context=context)
        return ret
    
    def case_cancel(self, cr, uid, ids, context=None):
        ret = super(task, self).case_cancel(cr, uid, ids, context=context)
        self.advance_childs_if_possible(cr, uid, ids, context=context)
        return ret
        
    def advance_childs_if_possible(self, cr, uid, ids, context=None):
        for it_task in self.browse(cr, uid, ids, context=context):
            if it_task.child_ids:
                for child_task in it_task.child_ids:
                    if child_task.state in ['pending']:
                        all_pred_in_final_state = True
                        if child_task.parent_ids:
                            for child_parent_task in child_task.parent_ids:
                                if child_parent_task.state not in ['cancelled', 'done']:
                                    all_pred_in_final_state = False
                                    break
                        if all_pred_in_final_state:
                            self.case_open(cr, uid, [child_task.id], context=context)
    
task()


class project(osv.osv):
    _inherit = 'project.project'
    
    _defaults = {
        'state': 'draft',
    }
    
    def _check_child_project(self, cr, uid, ids, context=None):
        if context == None:
            context = {}
        projects = self.browse(cr, uid, ids, context=context)
        for it_project in projects:
            if it_project.parent_ids:
                for parent in it_project.parent_ids:
                    if parent.state in ['open', 'pending']:
                        raise osv.except_osv(_("Warning !"),
                                     _("Predecessor project still open.\nPlease cancel or complete predecessor project first."))
        return True
    
    def set_done(self, cr, uid, ids, context=None):
        self._check_child_project(cr, uid, ids, context=context)
        ret = super(project, self).set_done(cr, uid, ids, context=context)
        self.advance_childs_if_possible(cr, uid, ids, context=context)
        self.subprojects_to_done(cr, uid, ids, context=context)
        return ret
    
    def set_cancel(self, cr, uid, ids, context=None):
        ret = super(project, self).set_cancel(cr, uid, ids, context=context)
        self.advance_childs_if_possible(cr, uid, ids, context=context)
        self.subprojects_to_cancel(cr, uid, ids, context=context)
        return ret
    
    def set_open(self, cr, uid, ids, context=None):
        ret = super(project, self).set_open(cr, uid, ids, context=context)
        self.advance_subprojects_if_possible(cr, uid, ids, context=context)
        self.advance_tasks_if_possible(cr, uid, ids, context=context)
        return ret
        
    def set_template(self, cr, uid, ids, context=None):
        ret = super(project, self).set_template(cr, uid, ids, context=context)
        self.subprojects_to_template(cr, uid, ids, context=context)
        return ret
    
    def set_pending(self, cr, uid, ids, context=None):
        ret = super(project, self).set_pending(cr, uid, ids, context=context)
        self.subprojects_to_pending(cr, uid, ids, context=context)
        return ret
        
    # Subprojects change of state
    def subprojects_to_template(self, cr, uid, ids, context=None):
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.project_child_ids:
                self.set_template(cr, uid, [sp.id for sp in it_project.project_child_ids], context=context)
    
    def subprojects_to_cancel(self, cr, uid, ids, context=None):
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.project_child_ids:
                self.set_cancel(cr, uid, [sp.id for sp in it_project.project_child_ids], context=context)
    
    def subprojects_to_pending(self, cr, uid, ids, context=None):
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.project_child_ids:
                self.set_pending(cr, uid, [sp.id for sp in it_project.project_child_ids], context=context)
    
    def subprojects_to_done(self, cr, uid, ids, context=None):
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.project_child_ids:
                self.set_done(cr, uid, [sp.id for sp in it_project.project_child_ids], context=context)
    # 
    
    def advance_childs_if_possible(self, cr, uid, ids, context=None):
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.child_ids:
                for child_project in it_project.child_ids:
                    if child_project.state in ['pending']:
                        all_pred_in_final_state = True
                        if child_project.parent_ids:
                            for child_parent_project in child_project.parent_ids:
                                if child_parent_project.state not in ['cancelled', 'close']:
                                    all_pred_in_final_state = False
                                    break
                        if all_pred_in_final_state:
                            self.set_open(cr, uid, [child_project.id], context=context)
    
    def advance_subprojects_if_possible(self, cr, uid, ids, context=None):
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.project_child_ids:
                for subproject in it_project.project_child_ids:
                    if subproject.state in ['draft']:
                        can_go_to_in_progress = True
                        if subproject.parent_ids:
                            for subproject_parent in subproject.parent_ids:
                                if subproject_parent.state in ['draft', 'open', 'pending']:
                                    can_go_to_in_progress = False
                                    break
                        if can_go_to_in_progress:
                            self.set_open(cr, uid, [subproject.id], context=context)
                        else:
                            self.set_pending(cr, uid, [subproject.id], context=context)
    
    def advance_tasks_if_possible(self, cr, uid, ids, context=None):
        task_obj = self.pool.get('project.task')
        for it_project in self.browse(cr, uid, ids, context=context):
            if it_project.tasks:
                for task in it_project.tasks:
                    if task.state in ['draft']:
                        can_go_to_in_progress = True
                        if task.parent_ids:
                            for task_parent in task.parent_ids:
                                if task_parent.state in ['draft', 'open', 'pending']:
                                    can_go_to_in_progress = False
                                    break
                        if can_go_to_in_progress:
                            task_obj.do_open(cr, uid, [task.id], context=context)
                        else:
                            task_obj.do_pending(cr, uid, [task.id], context=context)
    
    def set_draft(self, cr, uid, ids, context=None):
        if not ids:
            return False
        if not isinstance(ids, list):
            ids = [ids]
        
        task_obj = self.pool.get('project.task')
        task_type_obj = self.pool.get('project.task.type')
        
        for project in self.browse(cr, uid, ids, context=context):
            self.write(cr, uid, [project.id], {'state':'draft'}, context=context)
            self.set_draft(cr, uid, [subproject.id for subproject in project.project_child_ids], context=context)
            
            task_obj.do_draft(cr, uid, [task.id for task in project.tasks], context=context)
            
            stage = task_type_obj.search(cr, uid, [('state', '=', 'draft')], context=context)
            if isinstance(stage, list):
                stage = stage[0]
            task_obj.write(cr, uid, [task.id for task in project.tasks], {'stage_id': stage},context=context)
        return ids
    
project()




